@@ -510,3 +510,53 @@ process_theme <- function(theme) {
 
     return(getFromNamespace(parts[2], parts[1]))
 }
+
+
+#' check_palette
+#' Check if the palette can be properly used
+#' @param palette palette
+#' @param datas_name names of the split data
+#' @keywords internal
+#' @return named list containing palette names
+check_palette <- function(palette, datas_name) {
+    palette <- as.list(palette)
+    stopifnot("'palette' must be specified" = length(palette) > 0)
+    if (length(palette) == 1 && length(datas_name) > 1) {
+        palette <- rep(palette, length(datas_name))
+    }
+    if (length(palette) < length(datas_name)) {
+        stop("The length of 'palette' (", length(palette), ") is less than the number ",
+            "(", length(datas_name), ") of unique values in 'split_by'")
+    }
+    if (is.null(names(palette))) {
+        names(palette)[1:length(datas_name)] <- datas_name
+    } else if (length(setdiff(datas_name, names(palette))) > 0) {
+        stop("Values in 'split_by' (",
+            paste(setdiff(datas_name, names(palette)), collapse = ", "), ") ",
+            "have no corresponding palette assigned in 'palette'")
+    }
+    return(palette)
+}
+
+
+#' check_palcolor
+#' Check if the palcolor can be properly used
+#' @param palcolor palcolor
+#' @param datas_name names of the split data
+#' @keywords internal
+#' @return named list containing color names
+check_palcolor <- function(palcolor, datas_name) {
+    if (is.null(palcolor)) { return(NULL) }
+    # as.list() will turn c("red", "blue") into list("red", "blue")
+    # but we need list(c("red", "blue"))
+    if (!is.list(palcolor)) { palcolor <- list(palcolor) }
+    if (length(palcolor) == 1 && length(datas_name) > 1) {
+        palcolor <- rep(palcolor, length(datas_name))
+    }
+    if (is.null(names(palcolor))) {
+        names(palcolor)[1:length(datas_name)] <- datas_name
+    }
+    # It's okay that some split_by values have no corresponding palcolor
+    # because they can be NULL
+    return(palcolor)
+}
